@@ -30,6 +30,18 @@ def abort(message):
     raise DistutilsSetupError(message)
 
 
+def get_pip_config(key, default=None):
+    """Configured pip key, if available"""
+    try:
+        from pip._internal.configuration import Configuration
+        cc = Configuration(isolated=False)
+        cc.load()
+        return cc.get_value(key)
+
+    except (Exception, ImportError):
+        return default
+
+
 def MetaCommand(cls):
     """Decorator allowing for less boilerplate in our commands"""
     return setupmeta.MetaDefs.register_command(cls)
@@ -45,6 +57,82 @@ def longest_line(lines, maximum=70):
     return min(longest, maximum)
 
 
+@MetaCommand
+class UberEggCommand(setuptools.Command):
+    """pack all dependencies as eggs"""
+
+    user_options = [
+        ('dist=', 'd', "directory to put eggs in"),
+        ("requirements=", "r", "grab egg for all dependencies listed in given requirements file"),
+    ]
+
+    def initialize_options(self):
+        self.dist = None
+        self.requirements = None
+
+    def _get_requirements(self):
+        if self.requirements:
+            from setupmeta.content import load_list
+
+            raw = load_list(self.requirements) or []
+            reqs = [r for r in raw if r and not r.startswith("-")]
+            setupmeta.trace("Extracted %s/%s reqs from %s" % (len(raw), len(reqs), self.requirements))
+            return reqs
+
+    def run(self):
+        """
+        Grab an egg for each requirement, if any.
+        This is useful to mimic spark's "uber-jars" (https://spark.apache.org/docs/latest/index.html)
+        """
+        if self.dist is None:
+            self.dist = "dist"
+
+        if self.requirements is None:
+            self.requirements = "requirements.txt"
+
+        egg_target = os.path.abspath(self.dist)
+        if not os.path.isdir(egg_target):
+            os.mkdir(egg_target)
+
+        reqs = self._get_requirements()
+        print("%s dependencies in %s" % (len(reqs), self.requirements))
+        if reqs:
+            with setupmeta.temp_resource():
+                easy_install = os.path.join(os.path.dirname(sys.executable), "easy_install")
+                common_args = ["-zax", "-d", "."]
+                index_url = get_pip_config("global.index-url")
+                if index_url:
+                    common_args.append("-i")
+                    common_args.append(index_url)
+
+                env = dict(os.environ)
+                env["PYTHONPATH"] = "."
+                for req in reqs:
+                    args = common_args[:]
+                    args.append(req)
+                    setupmeta.run_program(easy_install, *args, env=env)
+
+                eggs = [f for f in os.listdir(".") if f.endswith(".egg")]
+                print("Fetched %s eggs" % len(eggs))
+                for name in eggs:
+                    dest = os.path.join(egg_target, name)
+                    if os.path.exists(dest):  # pragma: no cover
+                        print("%s/%s already exists, skipping" % (self.dist, name))
+                        continue
+
+                    source = name
+                    action = "Grabbed"
+                    if os.path.isdir(source):
+                        # This disregards any `zip_safe=False` setting, this could be wrong
+                        # However, most libraries don't specify this, even though their project is zip-safe...
+                        # Spark users seem to get by with eggs just fine, so forcing a zip here
+                        action = "Force-zipped"
+                        source = shutil.make_archive(name, "zip", source)
+
+                    print("%s %s" % (action, name))
+                    shutil.copy2(source, dest)
+
+
 @MetaCommand
 class CheckCommand(check):
     """Perform checks on the package"""
